@@ -1,19 +1,50 @@
 //!
-//! A disk-storage replacement for the pure in-memory BTreeMap.
+//! A `BTreeMap`-like structure but storing data in disk.
+//!
+//! NOTE:
+//! - Both keys and values will be encoded in this structure
+//!     - Keys will be encoded by `KeyEnDeOrdered`
+//!     - Values will be encoded by some `serde`-like methods
+//! - It's your duty to ensure that the encoded key keeps a same order with the original key
+//!
+//! # Examples
+//!
+//! ```
+//! use vsdb::MapxOrd;
+//!
+//! let mut l = MapxOrd::new();
+//!
+//! l.insert(1, 0);
+//! l.insert_ref(&1, &0);
+//! l.insert(2, 0);
+//!
+//! l.iter().for_each(|(k, v)| {
+//!     assert!(k >= 1);
+//!     assert_eq!(v, 0);
+//! });
+//!
+//! l.remove(&2);
+//! assert_eq!(l.len(), 1);
+//!
+//! l.clear();
+//! assert_eq!(l.len(), 0);
+//! ```
 //!
-
-mod backend;
 
 #[cfg(test)]
 mod test;
 
-use crate::common::{
-    ende::{KeyEnDeOrdered, SimpleVisitor, ValueEnDe},
-    InstanceCfg,
+use crate::{
+    basic::mapx_raw::{MapxRaw, MapxRawIter},
+    common::{
+        ende::{KeyEnDeOrdered, SimpleVisitor, ValueEnDe},
+        InstanceCfg,
+    },
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
 use std::{
+    marker::PhantomData,
     mem::ManuallyDrop,
     ops::{Bound, Deref, DerefMut, RangeBounds},
     result::Result as StdResult,
@@ -27,7 +58,9 @@ where
     K: KeyEnDeOrdered,
     V: ValueEnDe,
 {
-    inner: backend::MapxOrd<K, V>,
+    inner: MapxRaw,
+    _pd0: PhantomData<K>,
+    _pd1: PhantomData<V>,
 }
 
 impl<K, V> From<InstanceCfg> for MapxOrd<K, V>
@@ -37,7 +70,9 @@ where
 {
     fn from(cfg: InstanceCfg) -> Self {
         Self {
-            inner: backend::MapxOrd::from(cfg),
+            inner: MapxRaw::from(cfg),
+            _pd0: PhantomData,
+            _pd1: PhantomData,
         }
     }
 }
@@ -52,9 +87,8 @@ where
     }
 }
 
-///////////////////////////////////////////////
-// Begin of the self-implementation for MapxOrd //
-/*********************************************/
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
 impl<K, V> MapxOrd<K, V>
 where
@@ -65,7 +99,9 @@ where
     #[inline(always)]
     pub fn new() -> Self {
         MapxOrd {
-            inner: backend::MapxOrd::must_new(),
+            inner: MapxRaw::new(),
+            _pd0: PhantomData,
+            _pd1: PhantomData,
         }
     }
 
@@ -77,53 +113,39 @@ where
     /// Imitate the behavior of 'BTreeMap<_>.get(...)'
     #[inline(always)]
     pub fn get(&self, key: &K) -> Option<V> {
-        self.inner.get(key)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn get_ref_bytes_k(&self, key: &[u8]) -> Option<V> {
-        self.inner.get_ref_bytes_k(key)
+        self.inner
+            .get(&key.to_bytes())
+            .map(|v| <V as ValueEnDe>::decode(&v).unwrap())
     }
 
     /// Get the closest smaller value, include itself.
     #[inline(always)]
     pub fn get_le(&self, key: &K) -> Option<(K, V)> {
-        self.inner.get_le(key)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn get_le_ref_bytes_k(&self, key: &[u8]) -> Option<(K, V)> {
-        self.inner.get_le_ref_bytes_k(key)
+        self.inner.get_le(&key.to_bytes()).map(|(k, v)| {
+            (
+                pnk!(K::from_bytes(k)),
+                <V as ValueEnDe>::decode(&v).unwrap(),
+            )
+        })
     }
 
     /// Get the closest larger value, include itself.
     #[inline(always)]
     pub fn get_ge(&self, key: &K) -> Option<(K, V)> {
-        self.inner.get_ge(key)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn get_ge_ref_bytes_k(&self, key: &[u8]) -> Option<(K, V)> {
-        self.inner.get_ge_ref_bytes_k(key)
+        self.inner.get_ge(&key.to_bytes()).map(|(k, v)| {
+            (
+                pnk!(K::from_bytes(k)),
+                <V as ValueEnDe>::decode(&v).unwrap(),
+            )
+        })
     }
 
     /// Imitate the behavior of 'BTreeMap<_>.get_mut(...)'
     #[inline(always)]
     pub fn get_mut(&mut self, key: &K) -> Option<ValueMut<'_, K, V>> {
-        self.inner
-            .get(key)
-            .map(move |v| ValueMut::new(self, key.clone(), v))
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn get_mut_ref_bytes_k(&mut self, key: &[u8]) -> Option<ValueMut<'_, K, V>> {
-        self.inner
-            .get_ref_bytes_k(key)
-            .and_then(|v| K::from_slice(key).ok().map(|k| ValueMut::new(self, k, v)))
+        self.inner.get(&key.to_bytes()).map(|v| {
+            ValueMut::new(self, key.clone(), <V as ValueEnDe>::decode(&v).unwrap())
+        })
     }
 
     /// Imitate the behavior of 'BTreeMap<_>.len()'.
@@ -147,19 +169,21 @@ where
     #[inline(always)]
     #[allow(missing_docs)]
     pub fn insert_ref(&mut self, key: &K, value: &V) -> Option<V> {
-        self.inner.insert_ref(key, value)
+        self.inner
+            .insert(&key.to_bytes(), &value.encode())
+            .map(|v| <V as ValueEnDe>::decode(&v).unwrap())
     }
 
-    /// same as the funtion without the '_' prefix, but use bytes key
+    // used to support efficient versioned-implementations
     #[inline(always)]
-    pub fn insert_ref_bytes_k(&mut self, key: &[u8], value: &V) -> Option<V> {
-        self.inner.insert_ref_bytes_k(key, value)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key&value
-    #[inline(always)]
-    pub fn insert_ref_bytes_kv(&mut self, key: &[u8], value: &[u8]) -> Option<V> {
-        self.inner.insert_ref_bytes_kv(key, value)
+    pub(crate) fn insert_ref_encoded_value(
+        &mut self,
+        key: &K,
+        value: &[u8],
+    ) -> Option<V> {
+        self.inner
+            .insert(&key.to_bytes(), value)
+            .map(|v| <V as ValueEnDe>::decode(&v).unwrap())
     }
 
     /// Similar with `insert`, but ignore the old value.
@@ -171,19 +195,7 @@ where
     #[inline(always)]
     #[allow(missing_docs)]
     pub fn set_value_ref(&mut self, key: &K, value: &V) {
-        self.inner.set_value_ref(key, value);
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn set_value_ref_bytes_k(&mut self, key: &[u8], value: &V) {
-        self.inner.set_value_ref_bytes_k(key, value);
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key&value
-    #[inline(always)]
-    pub fn set_value_ref_bytes_kv(&mut self, key: &[u8], value: &[u8]) {
-        self.inner.set_value_ref_bytes_kv(key, value);
+        self.inner.insert(&key.to_bytes(), &value.encode());
     }
 
     /// Imitate the behavior of '.entry(...).or_insert(...)'
@@ -198,20 +210,13 @@ where
         EntryRef { key, hdr: self }
     }
 
-    #[inline(always)]
-    #[allow(missing_docs)]
-    pub fn entry_ref_bytes_key<'a>(
-        &'a mut self,
-        key: &'a [u8],
-    ) -> EntryRefBytesKey<'a, K, V> {
-        EntryRefBytesKey { key, hdr: self }
-    }
-
     /// Imitate the behavior of '.iter()'
     #[inline(always)]
     pub fn iter(&self) -> MapxOrdIter<K, V> {
         MapxOrdIter {
             iter: self.inner.iter(),
+            _pd0: PhantomData,
+            _pd1: PhantomData,
         }
     }
 
@@ -231,11 +236,11 @@ where
         let l = match bounds.start_bound() {
             Bound::Included(lo) => {
                 ll = lo.to_bytes();
-                Bound::Included(ll.as_slice())
+                Bound::Included(&ll[..])
             }
             Bound::Excluded(lo) => {
                 ll = lo.to_bytes();
-                Bound::Excluded(ll.as_slice())
+                Bound::Excluded(&ll[..])
             }
             Bound::Unbounded => Bound::Unbounded,
         };
@@ -244,28 +249,19 @@ where
         let h = match bounds.end_bound() {
             Bound::Included(hi) => {
                 hh = hi.to_bytes();
-                Bound::Included(hh.as_slice())
+                Bound::Included(&hh[..])
             }
             Bound::Excluded(hi) => {
                 hh = hi.to_bytes();
-                Bound::Excluded(hh.as_slice())
+                Bound::Excluded(&hh[..])
             }
             Bound::Unbounded => Bound::Unbounded,
         };
 
         MapxOrdIter {
             iter: self.inner.range((l, h)),
-        }
-    }
-
-    /// range(start..end)
-    #[inline(always)]
-    pub fn range_ref_bytes_k<'a, R: RangeBounds<&'a [u8]>>(
-        &'a self,
-        bounds: R,
-    ) -> MapxOrdIter<K, V> {
-        MapxOrdIter {
-            iter: self.inner.range(bounds),
+            _pd0: PhantomData,
+            _pd1: PhantomData,
         }
     }
 
@@ -284,37 +280,21 @@ where
     /// Check if a key is exists.
     #[inline(always)]
     pub fn contains_key(&self, key: &K) -> bool {
-        self.inner.contains_key(key)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn contains_key_ref_bytes_k(&self, key: &[u8]) -> bool {
-        self.inner.contains_key_ref_bytes_k(key)
+        self.inner.contains_key(&key.to_bytes())
     }
 
     /// Remove a <K, V> from mem and disk.
     #[inline(always)]
     pub fn remove(&mut self, key: &K) -> Option<V> {
-        self.inner.remove(key)
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn remove_ref_bytes_k(&mut self, key: &[u8]) -> Option<V> {
-        self.inner.remove_ref_bytes_k(key)
+        self.inner
+            .remove(&key.to_bytes())
+            .map(|v| <V as ValueEnDe>::decode(&v).unwrap())
     }
 
     /// Remove a <K, V> from mem and disk.
     #[inline(always)]
     pub fn unset_value(&mut self, key: &K) {
-        self.inner.unset_value(key);
-    }
-
-    /// same as the funtion without the '_' prefix, but use bytes key
-    #[inline(always)]
-    pub fn unset_value_ref_bytes_k(&mut self, key: &[u8]) {
-        self.inner.unset_value_ref_bytes_k(key);
+        self.inner.remove(&key.to_bytes());
     }
 
     /// Clear all data.
@@ -324,13 +304,8 @@ where
     }
 }
 
-/*******************************************/
-// End of the self-implementation for MapxOrd //
-/////////////////////////////////////////////
-
-//////////////////////////////////////////////////////////////////////////////////
-// Begin of the implementation of ValueMut(returned by `self.get_mut`) for MapxOrd //
-/********************************************************************************/
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
 /// Returned by `<MapxOrd>.get_mut(...)`
 #[derive(Debug)]
@@ -398,13 +373,8 @@ where
     }
 }
 
-/******************************************************************************/
-// End of the implementation of ValueMut(returned by `self.get_mut`) for MapxOrd //
-////////////////////////////////////////////////////////////////////////////////
-
-///////////////////////////////////////////////////
-// Begin of the implementation of Entry for MapxOrd //
-/*************************************************/
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
 /// Imitate the `btree_map/btree_map::Entry`.
 pub struct Entry<'a, K, V>
@@ -454,45 +424,18 @@ where
     }
 }
 
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
+
 #[allow(missing_docs)]
-pub struct EntryRefBytesKey<'a, K, V>
-where
-    K: KeyEnDeOrdered,
-    V: ValueEnDe,
-{
-    key: &'a [u8],
-    hdr: &'a mut MapxOrd<K, V>,
-}
-
-impl<'a, K, V> EntryRefBytesKey<'a, K, V>
-where
-    K: KeyEnDeOrdered,
-    V: ValueEnDe,
-{
-    /// Imitate the `btree_map/btree_map::Entry.or_insert(...)`.
-    pub fn or_insert_ref(self, default: &V) -> ValueMut<'a, K, V> {
-        if !self.hdr.contains_key_ref_bytes_k(self.key) {
-            self.hdr.set_value_ref_bytes_k(self.key, default);
-        }
-        pnk!(self.hdr.get_mut_ref_bytes_k(self.key))
-    }
-}
-
-/***********************************************/
-// End of the implementation of Entry for MapxOrd //
-/////////////////////////////////////////////////
-
-//////////////////////////////////////////////////
-// Begin of the implementation of Iter for MapxOrd //
-/************************************************/
-
-/// Iter over [MapxOrd](self::MapxOrd).
 pub struct MapxOrdIter<K, V>
 where
     K: KeyEnDeOrdered,
     V: ValueEnDe,
 {
-    iter: backend::MapxOrdIter<K, V>,
+    iter: MapxRawIter,
+    _pd0: PhantomData<K>,
+    _pd1: PhantomData<V>,
 }
 
 impl<K, V> Iterator for MapxOrdIter<K, V>
@@ -502,7 +445,12 @@ where
 {
     type Item = (K, V);
     fn next(&mut self) -> Option<Self::Item> {
-        self.iter.next()
+        self.iter.next().map(|(k, v)| {
+            (
+                pnk!(K::from_bytes(k)),
+                <V as ValueEnDe>::decode(&v).unwrap(),
+            )
+        })
     }
 }
 
@@ -512,17 +460,24 @@ where
     V: ValueEnDe,
 {
     fn next_back(&mut self) -> Option<Self::Item> {
-        self.iter.next_back()
+        self.iter.next_back().map(|(k, v)| {
+            (
+                pnk!(K::from_bytes(k)),
+                <V as ValueEnDe>::decode(&v).unwrap(),
+            )
+        })
     }
 }
 
-/**********************************************/
-// End of the implementation of Iter for MapxOrd //
-////////////////////////////////////////////////
+impl<K, V> ExactSizeIterator for MapxOrdIter<K, V>
+where
+    K: KeyEnDeOrdered,
+    V: ValueEnDe,
+{
+}
 
-///////////////////////////////////////////////////////////////////
-// Begin of the implementation of Serialize/Deserialize for MapxOrd //
-/*****************************************************************/
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
 impl<K, V> Serialize for MapxOrd<K, V>
 where
@@ -533,7 +488,9 @@ where
     where
         S: serde::Serializer,
     {
-        serializer.serialize_bytes(&self.get_instance_cfg().encode())
+        serializer.serialize_bytes(&<InstanceCfg as ValueEnDe>::encode(
+            &self.get_instance_cfg(),
+        ))
     }
 }
 
@@ -546,13 +503,11 @@ where
     where
         D: serde::Deserializer<'de>,
     {
-        deserializer.deserialize_bytes(SimpleVisitor).map(|meta| {
-            let meta = pnk!(<InstanceCfg as ValueEnDe>::decode(&meta));
-            MapxOrd::from(meta)
-        })
+        deserializer
+            .deserialize_bytes(SimpleVisitor)
+            .map(|cfg| MapxOrd::from(<InstanceCfg as ValueEnDe>::decode(&cfg).unwrap()))
     }
 }
 
-/***************************************************************/
-// End of the implementation of Serialize/Deserialize for MapxOrd //
-/////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
