@@ -17,8 +17,8 @@ fn basic_cases() {
             .map(|i: usize| (i.to_be_bytes(), i.to_be_bytes()))
             .for_each(|(i, b)| {
                 hdr_i.entry(&i).or_insert(&b);
-                assert_eq!(&hdr_i.get(&i).unwrap(), &i);
-                assert_eq!(&hdr_i.remove(&i).unwrap(), &b);
+                assert_eq!(&hdr_i.get(&i).unwrap()[..], &i[..]);
+                assert_eq!(&hdr_i.remove(&i).unwrap()[..], &b[..]);
                 assert!(hdr_i.get(&i).is_none());
                 assert!(hdr_i.insert(&i, &b).is_none());
                 assert!(hdr_i.insert(&i, &b).is_some());
@@ -34,12 +34,12 @@ fn basic_cases() {
     assert_eq!(cnt, reloaded.len());
 
     (0..cnt).map(|i: usize| i.to_be_bytes()).for_each(|i| {
-        assert_eq!(i.to_vec(), reloaded.get(&i).unwrap());
+        assert_eq!(&i[..], &reloaded.get(&i).unwrap()[..]);
     });
 
     (1..cnt).map(|i: usize| i.to_be_bytes()).for_each(|i| {
-        *pnk!(reloaded.get_mut(&i)) = i.to_vec();
-        assert_eq!(&reloaded.get(&i).unwrap(), &i);
+        *reloaded.get_mut(&i).unwrap() = i.to_vec().into_boxed_slice();
+        assert_eq!(&reloaded.get(&i).unwrap()[..], &i[..]);
         assert!(reloaded.contains_key(&i));
         assert!(reloaded.remove(&i).is_some());
         assert!(!reloaded.contains_key(&i));
@@ -56,12 +56,24 @@ fn basic_cases() {
 
     assert!(reloaded.range(&[][..]..&[1][..]).next().is_none());
     assert_eq!(
-        vec![4],
+        vec![4].into_boxed_slice(),
         reloaded.range(&[2][..]..&[10][..]).next().unwrap().1
     );
 
-    assert_eq!(vec![80], reloaded.get_ge(&[79]).unwrap().1);
-    assert_eq!(vec![80], reloaded.get_ge(&[80]).unwrap().1);
-    assert_eq!(vec![80], reloaded.get_le(&[80]).unwrap().1);
-    assert_eq!(vec![80], reloaded.get_le(&[100]).unwrap().1);
+    assert_eq!(
+        vec![80].into_boxed_slice(),
+        reloaded.get_ge(&[79]).unwrap().1
+    );
+    assert_eq!(
+        vec![80].into_boxed_slice(),
+        reloaded.get_ge(&[80]).unwrap().1
+    );
+    assert_eq!(
+        vec![80].into_boxed_slice(),
+        reloaded.get_le(&[80]).unwrap().1
+    );
+    assert_eq!(
+        vec![80].into_boxed_slice(),
+        reloaded.get_le(&[100]).unwrap().1
+    );
 }
