@@ -1,6 +1,30 @@
 //!
-//! A disk-storage replacement for the pure in-memory Vec.
+//! A disk-storage replacement for the in-memory Vec.
 //!
+//! NOTE:
+//! - Values will be encoded by some `serde`-like methods
+//!
+//! # Examples
+//!
+//! ```
+//! use vsdb::Vecx;
+//!
+//! let mut l = Vecx::new();
+//!
+//! l.push(1);
+//! for i in l.iter() {
+//!     assert_eq!(1, i);
+//! }
+//!
+//! l.pop();
+//! assert_eq!(l.len(), 0);
+//!
+//! l.insert(0, 1);
+//! assert_eq!(l.len(), 1);
+//!
+//! l.clear();
+//! assert_eq!(l.len(), 0);
+//! ```
 
 #[cfg(test)]
 mod test;
@@ -13,24 +37,18 @@ use crate::{
     },
 };
 use ruc::*;
-use std::cmp::Ordering;
+use std::{cmp::Ordering, result::Result as StdResult};
 
 /// To solve the problem of unlimited memory usage,
 /// use this to replace the original in-memory 'Vec'.
 ///
 /// - Each time the program is started, a new database is created
 #[derive(PartialEq, Eq, Debug)]
-pub struct Vecx<T>
-where
-    T: ValueEnDe,
-{
+pub struct Vecx<T: ValueEnDe> {
     inner: MapxOrd<usize, T>,
 }
 
-impl<T> From<InstanceCfg> for Vecx<T>
-where
-    T: ValueEnDe,
-{
+impl<T: ValueEnDe> From<InstanceCfg> for Vecx<T> {
     fn from(cfg: InstanceCfg) -> Self {
         Self {
             inner: MapxOrd::from(cfg),
@@ -38,23 +56,16 @@ where
     }
 }
 
-impl<T> Default for Vecx<T>
-where
-    T: ValueEnDe,
-{
+impl<T: ValueEnDe> Default for Vecx<T> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-///////////////////////////////////////////////
-// Begin of the self-implementation for Vecx //
-/*********************************************/
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
-impl<T> Vecx<T>
-where
-    T: ValueEnDe,
-{
+impl<T: ValueEnDe> Vecx<T> {
     /// Create an instance.
     #[inline(always)]
     pub fn new() -> Self {
@@ -204,59 +215,38 @@ where
     }
 }
 
-/*******************************************/
-// End of the self-implementation for Vecx //
-/////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
-//////////////////////////////////////////////////
-// Begin of the implementation of Iter for Vecx //
-/************************************************/
-
-/// Iter over [Vecx](self::Vecx).
-pub struct VecxIter<T>
-where
-    T: ValueEnDe,
-{
+#[allow(missing_docs)]
+pub struct VecxIter<T: ValueEnDe> {
     iter: MapxOrdIter<usize, T>,
 }
 
-impl<T> Iterator for VecxIter<T>
-where
-    T: ValueEnDe,
-{
+impl<T: ValueEnDe> Iterator for VecxIter<T> {
     type Item = T;
     fn next(&mut self) -> Option<Self::Item> {
         self.iter.next().map(|v| v.1)
     }
 }
 
-impl<T> DoubleEndedIterator for VecxIter<T>
-where
-    T: ValueEnDe,
-{
+impl<T: ValueEnDe> DoubleEndedIterator for VecxIter<T> {
     fn next_back(&mut self) -> Option<Self::Item> {
         self.iter.next_back().map(|v| v.1)
     }
 }
 
-/**********************************************/
-// End of the implementation of Iter for Vecx //
-////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
 
-///////////////////////////////////////////////////////////////////
-// Begin of the implementation of Serialize/Deserialize for Vecx //
-/*****************************************************************/
-
-impl<'a, T> serde::Serialize for Vecx<T>
-where
-    T: ValueEnDe,
-{
-    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
+impl<'a, T: ValueEnDe> serde::Serialize for Vecx<T> {
+    fn serialize<S>(&self, serializer: S) -> StdResult<S::Ok, S::Error>
     where
         S: serde::Serializer,
     {
-        let v = <InstanceCfg as ValueEnDe>::encode(&self.get_instance_cfg());
-        serializer.serialize_bytes(&v)
+        serializer.serialize_bytes(&<InstanceCfg as ValueEnDe>::encode(
+            &self.get_instance_cfg(),
+        ))
     }
 }
 
@@ -264,17 +254,15 @@ impl<'de, T> serde::Deserialize<'de> for Vecx<T>
 where
     T: ValueEnDe,
 {
-    fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
+    fn deserialize<D>(deserializer: D) -> StdResult<Self, D::Error>
     where
         D: serde::Deserializer<'de>,
     {
-        deserializer.deserialize_bytes(SimpleVisitor).map(|meta| {
-            let meta = pnk!(<InstanceCfg as ValueEnDe>::decode(&meta));
-            Vecx::from(meta)
-        })
+        deserializer
+            .deserialize_bytes(SimpleVisitor)
+            .map(|cfg| Vecx::from(<InstanceCfg as ValueEnDe>::decode(&cfg).unwrap()))
     }
 }
 
-/***************************************************************/
-// End of the implementation of Serialize/Deserialize for Vecx //
-/////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////
