@@ -1,12 +1,14 @@
 use crate::common::{
-    get_data_dir, vsdb_set_base_dir, BranchID, Engine, Prefix, PrefixBytes, VersionID,
-    PREFIX_SIZ, RESERVED_ID_CNT,
+    get_data_dir, vsdb_set_base_dir, BranchID, Engine, Prefix, PrefixBytes, RawKey,
+    RawValue, VersionID, PREFIX_SIZ, RESERVED_ID_CNT,
 };
 use ruc::*;
-use sled::{Config, Db, Iter, Mode, Tree};
+use sled::{Config, Db, IVec, Iter, Mode, Tree};
 use std::ops::{Bound, RangeBounds};
 
-const DATA_SET_NUM: u8 = 8;
+// the 'prefix search' in sled is just a global scaning,
+// use a relative larger number to sharding the `Tree` pressure.
+const DATA_SET_NUM: usize = 512;
 
 const META_KEY_BRANCH_ID: [u8; 1] = [u8::MAX - 1];
 const META_KEY_VERSION_ID: [u8; 1] = [u8::MAX - 2];
@@ -72,7 +74,11 @@ impl Engine for SledEngine {
 
     fn alloc_version_id(&self) -> VersionID {
         let ret = crate::parse_int!(
-            self.meta.get(META_KEY_BRANCH_ID).unwrap().unwrap().to_vec(),
+            self.meta
+                .get(META_KEY_VERSION_ID)
+                .unwrap()
+                .unwrap()
+                .to_vec(),
             VersionID
         );
         self.meta
@@ -81,8 +87,8 @@ impl Engine for SledEngine {
         ret
     }
 
-    fn area_count(&self) -> u8 {
-        self.areas.len() as u8
+    fn area_count(&self) -> usize {
+        self.areas.len()
     }
 
     fn flush(&self) {
@@ -94,6 +100,7 @@ impl Engine for SledEngine {
     fn iter(&self, area_idx: usize, meta_prefix: PrefixBytes) -> SledIter {
         SledIter {
             inner: self.areas[area_idx].scan_prefix(meta_prefix.as_slice()),
+            bounds: (Bound::Unbounded, Bound::Unbounded),
         }
     }
 
@@ -107,11 +114,11 @@ impl Engine for SledEngine {
         let l = match bounds.start_bound() {
             Bound::Included(lo) => {
                 b_lo.extend_from_slice(lo);
-                Bound::Included(b_lo)
+                Bound::Included(IVec::from(b_lo))
             }
             Bound::Excluded(lo) => {
                 b_lo.extend_from_slice(lo);
-                Bound::Excluded(b_lo)
+                Bound::Excluded(IVec::from(b_lo))
             }
             Bound::Unbounded => Bound::Unbounded,
         };
@@ -120,17 +127,18 @@ impl Engine for SledEngine {
         let h = match bounds.end_bound() {
             Bound::Included(hi) => {
                 b_hi.extend_from_slice(hi);
-                Bound::Included(b_hi)
+                Bound::Included(IVec::from(b_hi))
             }
             Bound::Excluded(hi) => {
                 b_hi.extend_from_slice(hi);
-                Bound::Excluded(b_hi)
+                Bound::Excluded(IVec::from(b_hi))
             }
             Bound::Unbounded => Bound::Unbounded,
         };
 
         SledIter {
-            inner: self.areas[area_idx].range((l, h)),
+            inner: self.areas[area_idx].scan_prefix(meta_prefix.as_slice()),
+            bounds: (l, h),
         }
     }
 
@@ -139,10 +147,13 @@ impl Engine for SledEngine {
         area_idx: usize,
         meta_prefix: PrefixBytes,
         key: &[u8],
-    ) -> Option<Vec<u8>> {
+    ) -> Option<RawValue> {
         let mut k = meta_prefix.to_vec();
         k.extend_from_slice(key);
-        self.areas[area_idx].get(k).unwrap().map(|iv| iv.to_vec())
+        self.areas[area_idx]
+            .get(k)
+            .unwrap()
+            .map(|iv| iv.to_vec().into_boxed_slice())
     }
 
     fn insert(
@@ -151,13 +162,13 @@ impl Engine for SledEngine {
         meta_prefix: PrefixBytes,
         key: &[u8],
         value: &[u8],
-    ) -> Option<Vec<u8>> {
+    ) -> Option<RawValue> {
         let mut k = meta_prefix.to_vec();
         k.extend_from_slice(key);
         self.areas[area_idx]
             .insert(k, value)
             .unwrap()
-            .map(|iv| iv.to_vec())
+            .map(|iv| iv.to_vec().into_boxed_slice())
     }
 
     fn remove(
@@ -165,36 +176,47 @@ impl Engine for SledEngine {
         area_idx: usize,
         meta_prefix: PrefixBytes,
         key: &[u8],
-    ) -> Option<Vec<u8>> {
+    ) -> Option<RawValue> {
         let mut k = meta_prefix.to_vec();
         k.extend_from_slice(key);
         self.areas[area_idx]
             .remove(k)
             .unwrap()
-            .map(|iv| iv.to_vec())
+            .map(|iv| iv.to_vec().into_boxed_slice())
     }
 }
 
 pub struct SledIter {
     inner: Iter,
+    bounds: (Bound<IVec>, Bound<IVec>),
 }
 
 impl Iterator for SledIter {
-    type Item = (Vec<u8>, Vec<u8>);
+    type Item = (RawKey, RawValue);
     fn next(&mut self) -> Option<Self::Item> {
-        self.inner
-            .next()
-            .map(|v| v.unwrap())
-            .map(|(ik, iv)| (ik[PREFIX_SIZ..].to_vec(), iv.to_vec()))
+        while let Some((k, v)) = self.inner.next().map(|i| i.unwrap()) {
+            if self.bounds.contains(&k) {
+                return Some((
+                    k[PREFIX_SIZ..].to_vec().into_boxed_slice(),
+                    v.to_vec().into_boxed_slice(),
+                ));
+            }
+        }
+        None
     }
 }
 
 impl DoubleEndedIterator for SledIter {
     fn next_back(&mut self) -> Option<Self::Item> {
-        self.inner
-            .next_back()
-            .map(|v| v.unwrap())
-            .map(|(ik, iv)| (ik[PREFIX_SIZ..].to_vec(), iv.to_vec()))
+        while let Some((k, v)) = self.inner.next_back().map(|i| i.unwrap()) {
+            if self.bounds.contains(&k) {
+                return Some((
+                    k[PREFIX_SIZ..].to_vec().into_boxed_slice(),
+                    v.to_vec().into_boxed_slice(),
+                ));
+            }
+        }
+        None
     }
 }
 
