@@ -40,7 +40,10 @@ pub mod versioned;
 
 pub use basic::mapx::Mapx;
 pub use basic::mapx_ord::MapxOrd;
+pub use basic::mapx_ord_rawkey::MapxOrdRawKey;
+pub use basic::mapx_ord_rawvalue::MapxOrdRawValue;
 pub use basic::mapx_raw::MapxRaw;
+pub use basic::single_value::SingleValue;
 pub use basic::vecx::Vecx;
 
 pub use versioned::mapx_raw::MapxRawVersioned;
